import 'package:flutter/material.dart';

import 'package:flutter_baidu_mapapi_base/flutter_baidu_mapapi_base.dart';
import 'package:flutter_baidu_mapapi_map/flutter_baidu_mapapi_map.dart';
import 'package:flutter_baidu_yingyan_trace/flutter_baidu_yingyan_trace.dart';

import 'package:flutter_yingyan_trace_demo/customWidgets/custom_button.dart';
import 'package:flutter_yingyan_trace_demo/customWidgets/map_appbar.dart';
import 'package:flutter_yingyan_trace_demo/customWidgets/text_field.dart';
import 'package:flutter_yingyan_trace_demo/constants.dart';
import 'package:flutter_yingyan_trace_demo/general/toast.dart';

/// 客户端理围栏
class LocalFencePage extends StatefulWidget {
  LocalFencePage({Key? key}) : super(key: key);

  @override
  _LocalFencePageState createState() => _LocalFencePageState();
}

class _LocalFencePageState extends State<LocalFencePage>
    with SingleTickerProviderStateMixin {
  TextEditingController _nameController = TextEditingController();
  TextEditingController _deNoiseController = TextEditingController();
  TextEditingController _radiusController = TextEditingController();
  late AnimationController _toolBarAnimationController;
  late Animation<Offset> _toolBarAnimation;
  bool _isCreateBtnSelected = false;
  BMFCoordinate? _fenceCenterCoordinate;
  BMFCircle? _circle;
  BMFMarker? _marker;
  late BMFMapController _mapController;
  List<BMFMarker> _markers = [];
  List<BMFCircle> _circles = [];

  @override
  void initState() {
    super.initState();

    ///
    _toolBarAnimationController =
        AnimationController(duration: Duration(milliseconds: 500), vsync: this);
    _toolBarAnimation = Tween(
      begin: Offset(0, 0),
      end: Offset(0, 1),
    ).animate(_toolBarAnimationController);
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      appBar: BMFAppBar(
        title: "客户端地理围栏",
        actions: [
          CustomButton(
            title: "查询",
            onTap: _onTapQueryFence,
          ),
          CustomButton(
            title: "创建",
            selectedTitle: "完成",
            selected: _isCreateBtnSelected,
            onTap: _onTapCreateFence,
          ),
        ],
      ),
      body: Stack(
        children: [
          _createMap(),
          _toolBar(),
        ],
      ),
    );
  }

  @override
  void dispose() {
    _nameController.dispose();
    _deNoiseController.dispose();
    _radiusController.dispose();
    _toolBarAnimationController.dispose();
    TraceController.shareInstance.release();

    super.dispose();
  }

  /// 查询
  void _onTapQueryFence() async {
    /// 客户端围栏查询
    QueryFenceListOption queryFenceListOption = QueryFenceListOption(
      tag: 1,
      serviceId: serviceID,
      monitoredPerson: entityName,
      fenceType: FenceType.local,
    );

    bool flag = await TraceController.shareInstance.queryFenceList(
        queryFenceListOption: queryFenceListOption,
        fenceCallback: FenceCallback(
            onQueryFenceListCallback: (QueryFenceListResult result) {
          print('-- 查询fence回调 result = ${result.toMap()}');

          showToast(msg: result.message, duration: 2);

          if (result.status != 0) return;

          /// 移除marker和circle
          if (_marker != null) {
            _mapController.removeMarker(_marker!);
          }
          if (_circle != null) {
            _mapController.removeOverlay(_circle!.Id);
          }
          _mapController.removeMarkers(_markers);
          _circles.forEach((element) {
            _mapController.removeOverlay(element.Id);
          });
          _markers.clear();
          _circles.clear();

          /// 围栏信息
          List<FenceInfo>? fenceInfos = result.fenceInfos;
          if (fenceInfos == null) return;

          fenceInfos.forEach((element) {
            /// marker
            BMFCoordinate coordinate = BMFCoordinate(
                element.circleFence!.center.latitude,
                element.circleFence!.center.longitude);
            BMFMarker marker = BMFMarker.icon(
                position: coordinate, icon: "images/general/icon_gcoding.png");
            _mapController.addMarker(marker);
            _markers.add(marker);

            /// circle
            BMFCircle circle = BMFCircle(
              center: coordinate,
              radius: element.circleFence!.radius,
              width: 0,
              fillColor: Colors.blue.withAlpha(100),
            );
            _mapController.addCircle(circle);
            _circles.add(circle);
          });

          /// 设置地图中心为第一个围栏中心
          if (_markers.length > 0) {
            _mapController.setCenterCoordinate(_markers[0].position, true);
          }
        }));

    print('-- 查询fence flag = $flag');
  }

  /// 创建
  void _onTapCreateFence() {
    if (!_isCreateBtnSelected) {
      setState(() {
        _isCreateBtnSelected = true;
      });
      _showToolBar();
    } else {
      if (_nameController.text.isEmpty) {
        showToast(msg: "请输入围栏名称");
        return;
      }

      if (_fenceCenterCoordinate == null) {
        showToast(msg: "请点击地图空白处以设置围栏中心");
        return;
      }

      if (_deNoiseController.text.isEmpty) {
        _deNoiseController.text = "0";
      }

      if (_radiusController.text.isEmpty) {
        _radiusController.text = "50";
      }

      _createFence();
      _hideToolBar();

      setState(() {
        _isCreateBtnSelected = false;
      });
    }
  }

  void _createFence() async {
    /// 客户端围栏
    CreateFenceOption createFenceOption =
        CreateFenceOption.buildLocalCircleOption(
      tag: 1,
      serviceId: serviceID,
      center: LatLng(
          _fenceCenterCoordinate!.latitude, _fenceCenterCoordinate!.longitude),
      radius: double.parse(_radiusController.text),
      fenceName: _nameController.text,
      monitoredPerson: entityName,
      deNoise: int.parse(_deNoiseController.text),
    );

    bool flag = await TraceController.shareInstance.createFence(
        createFenceOption: createFenceOption,
        fenceCallback:
            FenceCallback(onCreateFenceCallback: (CreateFenceResult result) {
          print('-- 创建围栏回调 result = ${result.toMap()}');

          showToast(msg: result.message);
          if (result.status != 0) return;

          _nameController.clear();
          _deNoiseController.clear();
          _radiusController.clear();
          _fenceCenterCoordinate = null;
        }));

    print('-- 创建围栏 flag = $flag');
  }

  ///
  void _showToolBar() {
    _toolBarAnimationController.forward();
  }

  ///
  void _hideToolBar() {
    _toolBarAnimationController.reverse();
  }

  /// 地图创建完成回调
  void _onBMFMapCreated(BMFMapController controller) {
    _mapController = controller;

    /// 点中底图空白处会回调接口
    controller.setMapOnClickedMapBlankCallback(
        callback: (BMFCoordinate coordinate) {
      _fenceCenterCoordinate = coordinate;

      /// 中心点marker
      if (_marker != null) {
        controller.removeMarker(_marker!);
      }

      BMFMarker marker = BMFMarker.icon(
          position: coordinate, icon: "images/general/icon_gcoding.png");
      controller.addMarker(marker);
      _marker = marker;

      /// 圆形overlay
      double radius = _radiusController.text.isNotEmpty
          ? double.parse(_radiusController.text)
          : 50;

      if (_circle != null) {
        controller.removeOverlay(_circle!.Id);
      }

      BMFCircle circle = BMFCircle(
        center: coordinate,
        radius: radius,
        width: 0,
        fillColor: Colors.blue.withAlpha(100),
      );
      controller.addCircle(circle);
      _circle = circle;

      controller.setCenterCoordinate(coordinate, true);
    });
  }

  /// 地图
  BMFMapWidget _createMap() {
    BMFMapOptions options = BMFMapOptions(
      zoomLevel: 16,
      center: BMFCoordinate(40.056056, 116.308102),
    );

    return BMFMapWidget(
      mapOptions: options,
      onBMFMapCreated: _onBMFMapCreated,
    );
  }

  ///
  Widget _toolBar() {
    return Positioned(
      top: -141,
      left: 0,
      right: 0,
      child: SlideTransition(
        position: _toolBarAnimation,
        child: Container(
          padding: EdgeInsets.only(left: 10, right: 10, bottom: 10),
          color: Colors.white,
          child: Column(
            children: [
              BMFTextField(
                controller: _nameController,
                title: "围栏名称：",
                placeholder: "字母数字汉字下划线的组合",
              ),
              BMFTextField(
                controller: _deNoiseController,
                title: "去噪精度：",
                placeholder: "默认0米",
              ),
              BMFTextField(
                controller: _radiusController,
                title: "围栏半径：",
                placeholder: "默认50米",
              ),
              SizedBox(
                height: 5,
              ),
              Text("在地图空白处戳点，以确定圆形圆心位置"),
            ],
          ),
        ),
      ),
    );
  }
}
