import 'package:flutter/material.dart';
import 'dart:io' show Platform;

import 'package:flutter_baidu_mapapi_base/flutter_baidu_mapapi_base.dart';
import 'package:flutter_baidu_yingyan_trace/flutter_baidu_yingyan_trace.dart';
import 'package:flutter_localizations/flutter_localizations.dart';
import 'package:flutter_yingyan_trace_demo/cachePage/cache_page.dart';
import 'package:flutter_yingyan_trace_demo/customWidgets/map_appbar.dart';
import 'package:flutter_yingyan_trace_demo/entityPage/entity_list_page.dart';
import 'package:flutter_yingyan_trace_demo/fencePage/server_fence_page.dart';
import 'package:flutter_yingyan_trace_demo/tracePage/trace_page.dart';
import 'package:permission_handler/permission_handler.dart';
import 'fencePage/local_fence_page.dart';
import 'constants.dart';
import 'history_track/history_track_page.dart';
import 'home_page.dart';

Future<void> main() async {
  /// 设置用户是否同意SDK隐私协议，默认false，since 2.1.0
  ///
  /// 设置为false时，将影响鹰眼SDK所有功能的使用
  ///
  /// 隐私政策官网链接：https://lbsyun.baidu.com/index.php?title=openprivacy
  /// isAgree 用户是否同意SDK隐私协议
  TraceSDK.setAgreePrivacy(true);

  runApp(new MyApp());

  requestPermission();

  // 设置鹰眼SDK的基础信息
  // 每次调用startService开启轨迹服务之前，可以重新设置这些信息。
  if (Platform.isIOS) {
    /// 鹰眼配置
    ServiceOption serviceOption = ServiceOption(
      ak: ak,
      mcode: mcode,
      serviceId: serviceID,
      keepAlive: false,
    );

    /// 设置SDK运行所需的基础信息，调用任何方法之前都需要先调用此方法
    /// true代表设置成功，false代表设置失败
    /// ios 独有
    bool flag =
        await TraceController.shareInstance.configServerInfo(serviceOption);
    print('--百度鹰眼服务配置 flag = $flag');

    /// 百度地图sdk初始化鉴权
    BMFMapSDK.setApiKeyAndCoordType(ak, BMF_COORD_TYPE.BD09LL);
  } else if (Platform.isAndroid) {
    /// Android 目前不支持接口设置Apikey,
    /// 请在主工程的Manifest文件里设置，详细配置方法请参考官网(https://lbsyun.baidu.com/)demo
    BMFMapSDK.setCoordType(BMF_COORD_TYPE.BD09LL);
  }
}

class MyApp extends StatefulWidget {
  @override
  _MyAppState createState() => _MyAppState();
}

class _MyAppState extends State<MyApp> {
  @override
  Widget build(BuildContext context) {
    return MaterialApp(
      localizationsDelegates: [
        GlobalMaterialLocalizations.delegate,
        GlobalWidgetsLocalizations.delegate,
      ],
      supportedLocales: [
        const Locale('zh', 'CH'),
        const Locale('en', 'US'),
      ],
      locale: Locale('zh'),
      home: Scaffold(
        appBar: BMFAppBar(
          title: '百度鹰眼flutter插件Demo',
          isBack: false,
        ),
        body: HomePage(),
      ),
      onGenerateRoute: (RouteSettings settings) {
        var routes = <String, WidgetBuilder>{
          '/entity_list_page': (context) => EntityListPage(),
          '/trace_page': (context) => TracePage(),
          '/history_track_page': (context) => HistoryTrackPage(),
          '/local_fence_page': (context) => LocalFencePage(),
          '/server_fence_page': (context) => ServerFencePage(),
          '/cache_page': (context) => CachePage(),
        };

        WidgetBuilder? builder = routes[settings.name];
        return builder != null
            ? MaterialPageRoute(builder: (context) => builder(context))
            : null;
      },
    );
  }
}

void requestPermission() async {
  Map<Permission, PermissionStatus> statuses =
      await [Permission.location].request();
  print(statuses);
}
