import 'dart:io';

import 'package:flutter/material.dart';
import 'package:flutter_baidu_mapapi_base/flutter_baidu_mapapi_base.dart';
import 'package:flutter_baidu_mapapi_map/flutter_baidu_mapapi_map.dart';
import 'package:flutter_baidu_yingyan_trace/flutter_baidu_yingyan_trace.dart';
import 'package:flutter_yingyan_trace_demo/customWidgets/map_appbar.dart';
import 'package:flutter_yingyan_trace_demo/customWidgets/map_raised_button.dart';
import 'package:flutter_yingyan_trace_demo/customWidgets/setting_button.dart';
import 'package:flutter_yingyan_trace_demo/constants.dart';
import 'package:flutter_yingyan_trace_demo/general/toast.dart';
import 'package:flutter_yingyan_trace_demo/tracePage/trace_setting_page.dart';

/// 轨迹追踪页
class TracePage extends StatefulWidget {
  @override
  _TracePageState createState() => _TracePageState();
}

class _TracePageState extends State<TracePage> {
  /// 开启鹰眼服务请求类
  Trace _trace = Trace(serviceId: serviceID, entityName: entityName);
  late BMFMapController _mapController;
  BMFMarker? _curLocation;

  @override
  void initState() {
    super.initState();

    if (Platform.isIOS) {
      _location();
    }
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      appBar: BMFAppBar(
        title: "轨迹追踪",
        actions: [
          SettingButton(
            onTap: _onTapSetting,
          )
        ],
      ),
      body: Stack(
        children: [
          _createMap(),
          _bottomToolBar(),
        ],
      ),
    );
  }

  /// 设置 tap
  void _onTapSetting() {
    Navigator.push(
        context, MaterialPageRoute(builder: (context) => TraceSettingPage()));
  }

  /// 定位
  void _location() async {
    /// 申请定位权限

    bool permissionFlag =
        await TraceController.shareInstance.requestLocalPermission();
    print('--申请定位权限 flag = $permissionFlag');

    /// 配置定位参数
    LocationOption locationOption = LocationOption();
    bool locationOptionFlag =
        await TraceController.shareInstance.configLocationInfo(locationOption);
    print('--配置定位参数 flag = $locationOptionFlag');
  }

  /// 开启服务 tap
  void _onTapService(bool isSelected) async {
    if (!isSelected) {
      // 开始
      bool flag = await TraceController.shareInstance.startTraceService(
        trace: _trace,
        traceCallback: TraceCallback(
          onStartTraceServiceCallBack: (TraceResult result) {
            print('--开启鹰眼服务回调 result = ${result.toMap()}');
            showToast(msg: '${result.toMap()}');
          },
          onPushCallBack: (PushResult result) {
            // 推送报警回调
            print('--推送报警回调 result = ${result.toMap()}');
            // 此处开发者对服务推送的报警信息进行处理
            showToast(msg: '${result.toMap()}');
          },
        ),
      );
      print('--开启鹰眼服务 flag = $flag');
    } else {
      // 停止
      bool flag = await TraceController.shareInstance.stopTraceService(
        trace: _trace,
        traceCallback: TraceCallback(
          onStopTraceServiceCallBack: (TraceResult result) {
            print('--停止鹰眼服务回调 result = ${result.toMap()}');
            showToast(msg: '${result.toMap()}');
          },
        ),
      );
      print('--停止鹰眼服务 flag = $flag');
    }
  }

  /// 开始采集
  void _onTapGather(bool isSelected) async {
    if (!isSelected) {
      // 开始
      bool flag = await TraceController.shareInstance.startGather(traceCallback:
          TraceCallback(onStartGatherCallBack: (GatherResult result) {
        print('--开始采集回调 result = ${result.toMap()}');
        showToast(msg: '${result.toMap()}');
      }));
      print('--开始采集 flag = $flag');
      if (Platform.isAndroid) {
        await TraceController.shareInstance.queryRealTimeLoc(
            realTimeLocationOption:
                RealTimeLocationOption(tag: 1, serviceId: serviceID),
            entityCallBack: EntityCallBack(onQueryRealTimeLocationCallBack:
                (RealTimeLocationResult realTimeLocationResult) {
              print('--实时定位回调 result = ${realTimeLocationResult.toMap()}');
              if (realTimeLocationResult.altitude == null ||
                  realTimeLocationResult.longitude == null) return;
              if (realTimeLocationResult.status == 0) {
                if (_curLocation == null) {
                  _curLocation = BMFMarker.icon(
                    position: BMFCoordinate(
                      realTimeLocationResult.latitude!,
                      realTimeLocationResult.longitude!,
                    ),
                    icon: "images/general/icon_cur_location.png",
                  );
                  _mapController.addMarker(_curLocation!);
                } else {
                  _curLocation!.updatePosition(BMFCoordinate(
                      realTimeLocationResult.latitude!,
                      realTimeLocationResult.longitude!));
                }
              }
            }));
      }
    } else {
      // 停止
      bool flag = await TraceController.shareInstance.stopGather(traceCallback:
          TraceCallback(onStopGatherCallBack: (GatherResult result) {
        print('--停止采集回调 result = ${result.toMap()}');
        showToast(msg: '${result.toMap()}');
      }));
      print('--停止采集 flag = $flag');
    }
  }

  /// 地图
  BMFMapWidget _createMap() {
    BMFMapOptions options = BMFMapOptions(
      zoomLevel: 16,
      center: BMFCoordinate(40.056056, 116.308102),
    );
    return BMFMapWidget(
      mapOptions: options,
      onBMFMapCreated: (BMFMapController controller) {
        _mapController = controller;
      },
    );
  }

  /// 开启服务 开启采集
  Widget _bottomToolBar() {
    return Positioned(
      left: 0,
      right: 0,
      bottom: 10,
      child: Container(
        child: Row(
          mainAxisAlignment: MainAxisAlignment.spaceEvenly,
          children: [
            BMFRaisedButton(
              title: "开启服务",
              selectedTitle: "停止服务",
              onPressed: _onTapService,
            ),
            BMFRaisedButton(
              title: "开始采集",
              selectedTitle: "停止采集",
              onPressed: _onTapGather,
            ),
          ],
        ),
      ),
    );
  }
}
